import os
import glob


def get_dataset_txt(dataset_path, dataset_file):
    file_data = glob.glob(os.path.join(dataset_path, "*.jpg"))
    with open(dataset_file, "w") as f:
        for file in file_data:
            f.writelines(f"{file}\n")




import metacv as mc
from ..model_zoo import load_model, run


class Segment(mc.Segment):
    def __init__(self,
                 model_path: str,
                 input_width: int,
                 input_height: int,
                 use_preprocess=True,
                 pad=None,
                 normal=None,
                 mean=None,
                 std=None,
                 swap=None,
                 confidence_thresh=None,
                 nms_thresh=None,
                 class_names=None,
                 device_id=0):
        super().__init__(model_path,
                         input_width,
                         input_height,
                         use_preprocess,
                         pad,
                         normal,
                         mean,
                         std,
                         swap,
                         confidence_thresh,
                         nms_thresh,
                         class_names)
        self.device_id = device_id
        self.model = None
        self.det_output = None
        self.mask_output = None
        self.input_names = None
        self.output_names = None
        self.bindings = None
        self.initialize_model()

    def initialize_model(self):
        self.model, self.input_names, self.output_names, self.bindings = load_model(self.model_path, self.device_id)

    def infer(self, image):
        # 由继承类实现模型推理
        batch_size = len(image) if isinstance(image, list) else 1
        outputs = run(image, self.model, self.input_names, self.output_names, self.bindings)
        mi = 4 + len(self.class_names)
        self.det_output = outputs[1].reshape((batch_size, mi + 32, -1))
        self.mask_output = outputs[0].reshape((batch_size, 32, -1))



import numpy as np

import tensorrt as trt
import pycuda.driver as cuda
import pycuda.autoinit


def load_model(model_path, device_id=0):
    input_names, output_names, bindings = [], [], []
    logger = trt.Logger(trt.Logger.WARNING)
    trt.init_libnvinfer_plugins(logger, '')
    with trt.Runtime(logger) as runtime, open(model_path, "rb") as f:
        engine = runtime.deserialize_cuda_engine(f.read())

    for e in engine:
        size = trt.volume(engine.get_binding_shape(e))
        dtype = trt.nptype(engine.get_binding_dtype(e))
        host_mem = cuda.pagelocked_empty(size, dtype)
        device_mem = cuda.mem_alloc(host_mem.nbytes)
        bindings.append(int(device_mem))
        if engine.binding_is_input(e):
            input_names.append({'host': host_mem, 'device': device_mem})
        else:
            output_names.append({'host': host_mem, 'device': device_mem})

    return engine, input_names, output_names, bindings


def run(images, model, input_names, output_names, bindings):
    input_tensor = np.array(images) if isinstance(images, list) else images[np.newaxis, :, :, :]

    stream = cuda.Stream()
    input_names[0]['host'] = np.ravel(input_tensor)
    for _input in input_names:
        cuda.memcpy_htod_async(_input['device'], _input['host'], stream)

    context = model.create_execution_context()
    context.execute_async_v2(bindings=bindings, stream_handle=stream.handle)

    for _output in output_names:
        cuda.memcpy_dtoh_async(_output['host'], _output['device'], stream)

    stream.synchronize()
    outputs = [out['host'] for out in output_names]

    return outputs



import cv2
import numpy as np
from metacv import Face as F
from .face_detection import FaceDetection
from .face_embedding import FaceEmbedding


class FaceRecognition:
    def __init__(self,
                 detection_model_path='models/face_detection.onnx',
                 recognition_model_path='models/face_recognition.onnx',
                 confidence_thresh=0.5,
                 nms_thresh=0.4,
                 class_names=['face'],
                 device_id=0):
        self.detector = FaceDetection(model_path=detection_model_path,
                                      input_width=480,
                                      input_height=640,
                                      use_preprocess=True,
                                      pad=True,
                                      normal=True,
                                      mean=[0.485, 0.456, 0.406],
                                      std=[0.229, 0.224, 0.225],
                                      swap=(2, 0, 1),
                                      confidence_thresh=confidence_thresh,
                                      nms_thresh=nms_thresh,
                                      class_names=class_names,
                                      device_id=device_id)
        self.recognizer = FaceEmbedding(model_path=recognition_model_path,
                                        input_width=112,
                                        input_height=112,
                                        use_preprocess=True,
                                        mean=[127.5, 127.5, 127.5],
                                        std=[127.5, 127.5, 127.5],
                                        swap=(2, 0, 1),
                                        device_id=device_id)

    def predict(self, image):
        total_faces = []
        total_bboxes, total_scores, total_kpss = self.detector.predict(image)
        for i in range(len(total_bboxes)):
            bboxes, scores, kpss = total_bboxes[i], total_scores[i], total_kpss[i]
            img = image[i] if isinstance(image, list) else image
            faces = []
            for bbox, score, kps in zip(bboxes, scores, kpss):
                face = F(bbox=bbox, kps=np.array(kps), score=score)
                align_img = F.norm_crop(img, face.kps)
                face.embedding = self.recognizer.predict(align_img)
                faces.append(face)
            total_faces.append(faces)

        return total_faces

    @staticmethod
    def show(image, faces):
        for face in faces:
            bbox, score, kps = face.bbox, face.score, face.kps
            x1, y1, x2, y2 = bbox
            cv2.rectangle(image, (x1, y1), (x2, y2), (255, 0, 0), 2)
            for k in kps:
                cv2.circle(image, (k[0], k[1]), 5, (0, 0, 255), -1)
            cv2.putText(image, '%.2f' % score,
                        ((x1 + x2) // 2, (y1 + y2) // 2), cv2.FONT_HERSHEY_COMPLEX_SMALL, 2, (0, 255, 0),
                        thickness=2)
        return image



import tensorrt as trt


class Quantization:
    def __init__(self,
                 model_path: str,  # onnx model name
                 precision="fp16",
                 workspace=12):
        self.model_path = model_path
        self.precision = precision
        self.workspace = workspace

    def convert(self, ):
        logger = trt.Logger(trt.Logger.INFO)
        builder = trt.Builder(logger)
        config = builder.create_builder_config()
        config.max_workspace_size = self.workspace * 1 << 30

        flag = 1 << int(trt.NetworkDefinitionCreationFlag.EXPLICIT_BATCH)
        network = builder.create_network(flag)

        parser = trt.OnnxParser(network, logger)
        if not parser.parse_from_file(self.model_path):
            raise RuntimeError(f'Failed to load ONNX file: {self.model_path}')

        inputs = [network.get_input(i) for i in range(network.num_inputs)]
        outputs = [network.get_output(i) for i in range(network.num_outputs)]
        for _input in inputs:
            print("Input '{}' with shape {} and dtype {}".format(_input.name, _input.shape, _input.dtype))
        for _output in outputs:
            print("Output '{}' with shape {} and dtype {}".format(_output.name, _output.shape, _output.dtype))

        if builder.platform_has_fast_fp16 and self.precision == "fp16":
            config.set_flag(trt.BuilderFlag.FP16)
        elif builder.platform_has_fast_int8 and self.precision == "int8":
            config.set_flag(trt.BuilderFlag.INT8)

        with builder.build_engine(network, config) as engine, \
                open(self.model_path.replace('.onnx', '.rt'), 'wb') as f:
            f.write(engine.serialize())
